@@ -11,6 +11,7 @@
 import sys
 import struct
 import unittest
+import datetime
 import io
 from collections import OrderedDict, namedtuple
 
@@ -116,6 +117,27 @@ single_test_vectors = [
     ["empty array", [], b"\x90"],
     # Empty Map
     ["empty map", {}, b"\x80"],
+    # 32-bit Timestamp
+    ["32-bit timestamp", datetime.datetime(1970, 1, 1, 0, 0, 0, 0, umsgpack._utc_tzinfo),
+        b"\xd6\xff\x00\x00\x00\x00"],
+    ["32-bit timestamp", datetime.datetime(2000, 1, 1, 10, 5, 2, 0, umsgpack._utc_tzinfo),
+        b"\xd6\xff\x38\x6d\xd1\x4e"],
+    # 64-bit Timestamp
+    ["64-bit timestamp", datetime.datetime(2000, 1, 1, 10, 5, 2, 1234, umsgpack._utc_tzinfo),
+        b"\xd7\xff\x00\x4b\x51\x40\x38\x6d\xd1\x4e"],
+    ["64-bit timestamp", datetime.datetime(2200, 1, 1, 10, 5, 2, 0, umsgpack._utc_tzinfo),
+        b"\xd7\xff\x00\x00\x00\x01\xb0\x9e\xa6\xce"],
+    ["64-bit timestamp", datetime.datetime(2200, 1, 1, 10, 5, 2, 1234, umsgpack._utc_tzinfo),
+        b"\xd7\xff\x00\x4b\x51\x41\xb0\x9e\xa6\xce"],
+    # 96-bit Timestamp
+    ["96-bit timestamp", datetime.datetime(1900, 1, 1, 10, 5, 2, 0, umsgpack._utc_tzinfo),
+        b"\xc7\x0c\xff\x00\x00\x00\x00\xff\xff\xff\xff\x7c\x56\x0f\x4e"],
+    ["96-bit timestamp", datetime.datetime(1900, 1, 1, 10, 5, 2, 1234, umsgpack._utc_tzinfo),
+        b"\xc7\x0c\xff\x00\x12\xd4\x50\xff\xff\xff\xff\x7c\x56\x0f\x4e"],
+    ["96-bit timestamp", datetime.datetime(3000, 1, 1, 10, 5, 2, 0, umsgpack._utc_tzinfo),
+        b"\xc7\x0c\xff\x00\x00\x00\x00\x00\x00\x00\x07\x91\x5f\x59\xce"],
+    ["96-bit timestamp", datetime.datetime(3000, 1, 1, 10, 5, 2, 1234, umsgpack._utc_tzinfo),
+        b"\xc7\x0c\xff\x00\x12\xd4\x50\x00\x00\x00\x07\x91\x5f\x59\xce"],
 ]
 
 composite_test_vectors = [
@@ -262,6 +284,9 @@ unpack_exception_test_vectors = [
     # Reserved code (0xc1)
     ["reserved code", b"\xc1",
         umsgpack.ReservedCodeException],
+    # Unsupported timestamp (unsupported data length)
+    ["unsupported timestamp", b"\xc7\x02\xff\xaa\xbb",
+        umsgpack.UnsupportedTimestampException],
     # Invalid string (non utf-8)
     ["invalid string", b"\xa1\x80",
         umsgpack.InvalidStringException],
@@ -297,9 +322,9 @@ float_precision_test_vectors = [
 CustomType = namedtuple('CustomType', ['x', 'y', 'z'])
 
 ext_handlers = {
-    complex: lambda obj: umsgpack.Ext(0x20, struct.pack("ff", obj.real, obj.imag)),
+    complex: lambda obj: umsgpack.Ext(0x20, struct.pack("<ff", obj.real, obj.imag)),
     CustomType: lambda obj: umsgpack.Ext(0x30, umsgpack.packb(list(obj))),
-    0x20: lambda ext: complex(*struct.unpack("ff", ext.data)),
+    0x20: lambda ext: complex(*struct.unpack("<ff", ext.data)),
     0x30: lambda ext: CustomType(*umsgpack.unpackb(ext.data)),
 }
 
@@ -309,6 +334,22 @@ ext_handlers_test_vectors = [
      b"\xd7\x30\x93\xc4\x03\x61\x62\x63\x7b\xc3"],
 ]
 
+override_ext_handlers = {
+    datetime.datetime:
+        lambda obj: umsgpack.Ext(0x40, obj.strftime("%Y%m%dT%H:%M:%S.%f").encode()),
+    -0x01:
+        lambda ext: ext,
+}
+
+override_ext_handlers_test_vectors = [
+    ["pack override",
+        datetime.datetime(2000, 1, 1, 10, 5, 2, 0, umsgpack._utc_tzinfo),
+        b'\xc7\x18@20000101T10:05:02.000000'],
+    ["unpack override",
+        umsgpack.Ext(-0x01, b"\x00\xbb\xcc\xdd\x01\x02\x03\x04\x05\x06\x07\x08"),
+        b'\xc7\x0c\xff\x00\xbb\xcc\xdd\x01\x02\x03\x04\x05\x06\x07\x08'],
+]
+
 # These are the only global variables that should be exported by umsgpack
 exported_vars_test_vector = [
     "Ext",
@@ -318,6 +359,7 @@ exported_vars_test_vector = [
     "UnsupportedTypeException",
     "InsufficientDataException",
     "InvalidStringException",
+    "UnsupportedTimestampException",
     "ReservedCodeException",
     "UnhashableKeyException",
     "DuplicateKeyException",
@@ -466,10 +508,7 @@ class TestUmsgpack(unittest.TestCase):
 
     def test_ext_exceptions(self):
         with self.assertRaises(TypeError):
-            _ = umsgpack.Ext(-1, b"")
-
-        with self.assertRaises(TypeError):
-            _ = umsgpack.Ext(128, b"")
+            _ = umsgpack.Ext(5.0, b"")
 
         with self.assertRaises(TypeError):
             _ = umsgpack.Ext(0, u"unicode string")
@@ -501,6 +540,26 @@ class TestUmsgpack(unittest.TestCase):
             packed = umsgpack.packb(obj, force_float_precision=precision)
             self.assertEqual(packed, data)
 
+    def test_pack_ext_override(self):
+        # Test overridden packing of datetime.datetime
+        (name, obj, data) = override_ext_handlers_test_vectors[0]
+        obj_repr = repr(obj)
+        print("\tTesting %s: object %s" %
+              (name, obj_repr if len(obj_repr) < 24 else obj_repr[0:24] + "..."))
+
+        packed = umsgpack.packb(obj, ext_handlers=override_ext_handlers)
+        self.assertEqual(packed, data)
+
+    def test_unpack_ext_override(self):
+        # Test overridden unpacking of Ext type -1
+        (name, obj, data) = override_ext_handlers_test_vectors[1]
+        obj_repr = repr(obj)
+        print("\tTesting %s: object %s" %
+              (name, obj_repr if len(obj_repr) < 24 else obj_repr[0:24] + "..."))
+
+        unpacked = umsgpack.unpackb(data, ext_handlers=override_ext_handlers)
+        self.assertEqual(unpacked, obj)
+
     def test_streaming_writer(self):
         # Try first composite test vector
         (_, obj, data) = composite_test_vectors[0]
@@ -519,7 +578,7 @@ class TestUmsgpack(unittest.TestCase):
         exported_vars = list(filter(lambda x: not x.startswith("_"),
                                     dir(umsgpack)))
         # Ignore imports
-        exported_vars = list(filter(lambda x: x != "struct" and x != "collections" and x !=
+        exported_vars = list(filter(lambda x: x != "struct" and x != "collections" and x != "datetime" and x !=
                                     "sys" and x != "io" and x != "xrange", exported_vars))
 
         self.assertTrue(len(exported_vars) == len(exported_vars_test_vector))
